'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

var _axis = require('../utilities/axis');

var _axis2 = _interopRequireDefault(_axis);

var _axisLabel = require('../utilities/axisLabel');

var _axisLabel2 = _interopRequireDefault(_axisLabel);

var _grid = require('../utilities/grid');

var _grid2 = _interopRequireDefault(_grid);

var _legend = require('../utilities/legend');

var _legend2 = _interopRequireDefault(_legend);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var BarGraph = function (_React$Component) {
  _inherits(BarGraph, _React$Component);

  function BarGraph(props) {
    _classCallCheck(this, BarGraph);

    var _this = _possibleConstructorReturn(this, (BarGraph.__proto__ || Object.getPrototypeOf(BarGraph)).call(this, props));

    _this.updateSize = function () {
      var node = _reactDom2.default.findDOMNode(_this);
      var parentWidth = node.offsetWidth;
      parentWidth < _this.props.width ? _this.setState({ width: parentWidth }) : _this.setState({ width: _this.props.width });
    };

    _this.updateSize = _this.updateSize.bind(_this);
    _this.state = {
      width: _this.props.width,
      data: []
    };
    return _this;
  }

  _createClass(BarGraph, [{
    key: 'componentWillMount',
    value: function componentWillMount() {
      window.addEventListener('resize', this.updateSize, false);
      this.setState({ width: this.props.width });
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.reloadBarData();
      this.repaintComponent();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      window.removeEventListener('resize', this.updateSize, false);
    }
  }, {
    key: 'repaintComponent',
    value: function repaintComponent() {
      var forceResize = this.updateSize;
      function onRepaint(callback) {
        setTimeout(function () {
          window.requestAnimationFrame(callback);
        }, 0);
      }
      onRepaint(forceResize);
    }
  }, {
    key: 'stackType',
    value: function stackType() {
      var tempArray = [];
      if (typeof this.props.yMax === "number") {
        tempArray.push(this.props.yMax);
      } else {
        var d = this.stacked;
        for (var i in d) {
          for (var j in d[i]) {
            if (this.props.barChartType === "side") {
              tempArray.push(d[i][j].y);
            } else {
              tempArray.push(d[i][j].y + d[i][j].y0);
            }
          }
        }
      }
      return tempArray;
    }
  }, {
    key: 'createChart',
    value: function createChart(_self) {
      if (this.props.colors) {
        this.color = _d2.default.scale.ordinal().range(this.props.colors);
      } else {
        this.color = _d2.default.scale.category10();
      }

      var xLabelHeightOffset = 0;
      var yLabelWidthOffset = 0;

      {
        this.props.xAxisLabel ? xLabelHeightOffset = 30 : null;
      }
      {
        this.props.yAxisLabel ? yLabelWidthOffset = 20 : null;
      }

      {
        this.props.yAxisPercent ? yLabelWidthOffset = yLabelWidthOffset + 10 : null;
      }

      // Width of graph
      this.w = this.state.width - (this.props.margin.left + this.props.margin.right + yLabelWidthOffset);

      // Height of graph
      this.h = this.props.height - (this.props.margin.top + this.props.margin.bottom + xLabelHeightOffset);

      this.stacked = _d2.default.layout.stack()(_self.props.keys.map(function (key) {
        return _self.state.data.map(function (d) {
          return { x: d[_self.props.xDataKey], y: d[key] };
        });
      }));

      // X0 axis scale
      this.x0Scale = _d2.default.scale.ordinal().rangeRoundBands([0, this.w], this.props.groupSpacing).domain(this.stacked[0].map(function (d) {
        return d.x;
      }));

      // X1 axis scale
      this.x1Scale = _d2.default.scale.ordinal().rangeRoundBands([0, this.x0Scale.rangeBand()], this.props.individualSpacing).domain(this.props.keys.map(function (d) {
        return d;
      }));

      // Y axis scale
      this.yScale = _d2.default.scale.linear().rangeRound([this.h, 0]).domain([0, _d2.default.max(this.stackType())]).nice();

      if (this.props.yAxisPercent) {
        this.yAxis = _d2.default.svg.axis().scale(this.yScale).orient('left').tickFormat(function (x) {
          return x + '%';
        }).ticks(5);
      } else {
        this.yAxis = _d2.default.svg.axis().scale(this.yScale).orient('left').ticks(5);
      }

      this.xAxis = _d2.default.svg.axis().scale(this.x0Scale).orient('bottom').ticks(this.state.data.length);

      this.yGrid = _d2.default.svg.axis().scale(this.yScale).orient('left').ticks(5).tickSize(-this.w, 0, 0).tickFormat("");

      this.transform = 'translate(' + (this.props.margin.left + yLabelWidthOffset) + ',' + this.props.margin.top + ')';
    }
  }, {
    key: 'reloadBarData',
    value: function reloadBarData() {
      var data = this.props.data;
      this.setState({ data: data });
    }
  }, {
    key: 'render',
    value: function render() {
      this.createChart(this);

      var _self = this;

      var bars = this.stacked.map(function (data, i) {
        var rects = void 0;
        if (_self.props.barChartType === "side") {
          rects = data.map(function (d, j) {
            return _react2.default.createElement('rect', {
              x: _self.x0Scale(d.x) + i * (_self.x0Scale.rangeBand() / _self.stacked.length),
              y: _self.h - (_self.yScale(d.y0) - _self.yScale(d.y + d.y0)),
              fill: _self.color(i),
              height: _self.yScale(d.y0) - _self.yScale(d.y + d.y0),
              width: _self.x1Scale.rangeBand(),
              key: j });
          });
        } else {
          rects = data.map(function (d, j) {
            return _react2.default.createElement('rect', {
              x: _self.x0Scale(d.x),
              y: _self.yScale(d.y + d.y0),
              fill: _self.color(i),
              height: _self.yScale(d.y0) - _self.yScale(d.y + d.y0),
              width: _self.x0Scale.rangeBand(),
              key: j });
          });
        }

        return _react2.default.createElement(
          'g',
          { key: i },
          rects
        );
      });

      var customClassName = "";

      if (this.props.chartClassName) {
        customClassName = " " + this.props.chartClassName;
      }

      var legend = [];
      var legendType = this.props.legendValues ? this.props.legendValues : this.props.keys;

      if (this.props.legend) {
        legendType.forEach(function (value, i) {
          var legendObj = {};
          legendObj[_self.props.labelKey] = value;
          legend[i] = legendObj;
        });
      }

      return _react2.default.createElement(
        'div',
        null,
        this.props.title && _react2.default.createElement(
          'h3',
          null,
          this.props.title
        ),
        _react2.default.createElement(
          'svg',
          { className: "rd3r-chart rd3r-bar-graph" + customClassName, id: this.props.chartId, width: this.state.width, height: this.props.height },
          _react2.default.createElement(
            'g',
            { transform: this.transform },
            _react2.default.createElement(_grid2.default, { h: this.h, grid: this.yGrid, gridType: 'y' }),
            _react2.default.createElement(_axis2.default, { h: this.h, axis: this.yAxis, axisType: 'y' }),
            _react2.default.createElement(_axis2.default, { h: this.h, axis: this.xAxis, axisType: 'x' }),
            this.props.xAxisLabel && _react2.default.createElement(_axisLabel2.default, { key: 0, h: this.h, w: this.w, axisLabel: this.props.xAxisLabel, axisType: 'x' }),
            this.props.yAxisLabel && _react2.default.createElement(_axisLabel2.default, { key: 1, h: this.h, w: this.w, axisLabel: this.props.yAxisLabel, axisType: 'y', padding: this.props.yAxisPercent ? 15 : 0 }),
            bars
          )
        ),
        this.props.legend && _react2.default.createElement(_legend2.default, { data: legend, labelKey: this.props.labelKey, colors: this.color })
      );
    }
  }]);

  return BarGraph;
}(_react2.default.Component);

BarGraph.propTypes = {
  title: _react2.default.PropTypes.string,
  width: _react2.default.PropTypes.number,
  height: _react2.default.PropTypes.number,
  chartId: _react2.default.PropTypes.string,
  chartClassName: _react2.default.PropTypes.string,
  colors: _react2.default.PropTypes.array,
  barChartType: _react2.default.PropTypes.oneOf(['stack', 'side']),
  groupSpacing: _react2.default.PropTypes.number,
  individualSpacing: _react2.default.PropTypes.number,
  data: _react2.default.PropTypes.array.isRequired,
  labelKey: _react2.default.PropTypes.string,
  xDataKey: _react2.default.PropTypes.string.isRequired,
  xAxisLabel: _react2.default.PropTypes.string,
  yAxisLabel: _react2.default.PropTypes.string,
  yAxisPercent: _react2.default.PropTypes.bool,
  legend: _react2.default.PropTypes.bool,
  keys: _react2.default.PropTypes.array.isRequired,
  legendValues: _react2.default.PropTypes.array,
  margin: _react2.default.PropTypes.object,
  yMax: _react2.default.PropTypes.number
};

BarGraph.defaultProps = {
  width: 1920,
  height: 400,
  barChartType: "stack",
  groupSpacing: .3,
  individualSpacing: .5,
  legend: true,
  labelKey: "label",
  margin: {
    top: 10,
    right: 40,
    bottom: 30,
    left: 40
  }
};

exports.default = BarGraph;