'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ToolTip = function ToolTip(_ref) {
  var tooltip = _ref.tooltip,
      bgStyle = _ref.bgStyle,
      chartWidth = _ref.chartWidth,
      margin = _ref.margin,
      xAxis = _ref.xAxis,
      xValue = _ref.xValue,
      yValue = _ref.yValue;

  var displayType = "none";
  var transform = "";
  var x = 0;
  var xOffset = 0;
  var y = 0;
  var xAxisPadding = xAxis ? 0 : 15;
  var width = 150;
  var height = 70;
  var transformText = 'translate(' + width / 2 + ',' + (height / 2 - 5) + ')';
  var transformArrow = "";

  if (tooltip.display === true) {
    x = tooltip.pos.x;
    y = tooltip.pos.y;
    displayType = "block";

    if (margin) {
      var pointLocation = tooltip.pos.x + width / 2 + margin.right + margin.left;
      if (pointLocation > chartWidth) {
        xOffset = pointLocation - chartWidth - xAxisPadding;
      } else if (tooltip.pos.x < width / 2) {
        xOffset = -(width / 2 - tooltip.pos.x - 15);
      }
    }

    if (y > height) {
      transform = 'translate(' + (x - width / 2 - xOffset) + ',' + (y - height - 20) + ')';
      transformArrow = 'translate(' + (width / 2 - 10 + xOffset) + ',' + (height - 1) + ')';
    } else if (y < height) {
      transform = 'translate(' + (x - width / 2 - xOffset) + ',' + (Math.round(y) + 20) + ')';
      transformArrow = 'translate(' + (width / 2 - 10 + xOffset) + ',' + 0 + ') rotate(180,10,0)';
    }
  }

  return _react2.default.createElement(
    'g',
    { transform: transform, style: { display: displayType } },
    _react2.default.createElement('rect', { className: 'shadow', width: width, height: height, rx: '5', ry: '5', fill: bgStyle, opacity: '.9' }),
    _react2.default.createElement('polygon', { className: 'shadow', points: '0,0  20,0  10,10', transform: transformArrow, fill: bgStyle, opacity: '.9' }),
    _react2.default.createElement(
      'text',
      { transform: transformText },
      _react2.default.createElement(
        'tspan',
        { x: '0', textAnchor: 'middle', fontSize: '15px', fill: '#ffffff' },
        xValue + tooltip.data.key
      ),
      _react2.default.createElement(
        'tspan',
        { x: '0', textAnchor: 'middle', dy: '25', fontSize: '20px', fill: '#ffffff' },
        yValue + tooltip.data.value
      )
    )
  );
};

ToolTip.propTypes = {
  tooltip: _react2.default.PropTypes.object,
  bgStyle: _react2.default.PropTypes.string,
  // textStyle1: React.PropTypes.string,
  // textStyle2: React.PropTypes.string,
  chartWidth: _react2.default.PropTypes.number,
  margin: _react2.default.PropTypes.object,
  xAxis: _react2.default.PropTypes.bool,
  xValue: _react2.default.PropTypes.string,
  yValue: _react2.default.PropTypes.string
};

ToolTip.defaultProps = {
  bgStyle: "#000000"
};

exports.default = ToolTip;